//
//  AM_ApplTableDataSource.m
//  GlowWorm
//
//  Created by Curtis Jones on 2006.12.23.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <netdb.h>

#import "AM_ApplTableDataSource.h"
#import <Common.h>
#import <Logger.h>





@implementation AM_ApplTableDataSource





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark initalization functions





- (id)init
{
	self = [super init];
	
	if (self) {
		mAppls = [[NSMutableArray arrayWithCapacity:50] retain];
		mApplsByName = [[NSMutableDictionary dictionaryWithCapacity:50] retain];
		mSockCount = 0;
	}
	
	return self;
}

- (void)dealloc
{
	if (mAppls) {
		[mAppls release];
		mAppls = nil;
	}
	
	if (mApplsByName) {
		[mApplsByName release];
		mApplsByName = nil;
	}
	
	[super dealloc];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark accessor functions (add/del)





- (void)addSock:(GlowWormSock *)aSock
{
	AM_ApplObject * aObj = nil;
	NSString * aStr = [aSock getApplication];
	
	if (aStr == nil || [aStr length] == 0 || [aStr hasPrefix:@"<Failed"]) {
		if (GWS_CONN_DIR_INCOMING == [[aSock getDirection] unsignedLongValue]) {
			struct servent * srvc = getservbyport(htons([[aSock getLocalPort] intValue]), "tcp");
			
			if (srvc != NULL)
				aStr = [NSString stringWithCString:srvc->s_name];
			else
				aStr = [NSString stringWithFormat:@"%@", [aSock getLocalPort]];
		}
		else
			aStr = @"Unknown";
	}
	else
	{
		NSRange aRange = [aStr rangeOfString:@".app/"];
		
		if (aRange.location != NSNotFound) {
			aRange.length = aRange.location;
			aRange.location = 0;
			aStr = [[aStr substringWithRange:aRange] retain];
		}
	}
	
	@synchronized (mAppls)
	{
		if (nil == (aObj = [[mApplsByName objectForKey:aStr] retain])) {
			aObj = [[AM_ApplObject alloc] initWithName:aStr origName:[aSock getApplication] andCount:1];
			[mApplsByName setObject:aObj forKey:aStr];
			[mAppls addObject:aObj];
		}
		else
			[aObj incrementCount];
	}
	
	[aObj release];
//[aStr release];
	
	mSockCount++;
}

- (void)delSock:(GlowWormSock *)aSock
{
	AM_ApplObject * aObj = nil;
	NSString * aStr = [aSock getApplication];
	
	if (aStr == nil || [aStr length] == 0 || [aStr hasPrefix:@"<Failed"]) {
		if (GWS_CONN_DIR_INCOMING == [[aSock getDirection] unsignedLongValue]) {
			struct servent * srvc = getservbyport(htons([[aSock getLocalPort] intValue]), "tcp");
			
			if (srvc != NULL)
				aStr = [NSString stringWithCString:srvc->s_name];
			else
				aStr = [NSString stringWithFormat:@"%@", [aSock getLocalPort]];
		}
		else
			aStr = @"Unknown";
	}
	else
	{
		NSRange aRange = [aStr rangeOfString:@".app/"];
		
		if (aRange.location != NSNotFound) {
			aRange.length = aRange.location;
			aRange.location = 0;
			aStr = [[aStr substringWithRange:aRange] retain];
		}
	}
	
	if (nil == (aObj = [mApplsByName objectForKey:aStr]))
		return;
	
	[aObj decrementCount];
	mSockCount--;
}

- (void)delAll
{
	@synchronized (mAppls)
	{
		[mAppls removeAllObjects];
		[mApplsByName removeAllObjects];
		
		mSockCount = 0;
	}
}

- (int)delOldAppls
{
	int i, count = 0;
	struct timeval tv;
	
	@synchronized (mAppls)
	{
		gettimeofday(&tv, NULL);
		
		for (i = 0; i < [mAppls count]; ++i) {
			AM_ApplObject * aObj = [mAppls objectAtIndex:i];
			
			if ([aObj count] != 0 || tv.tv_sec < [aObj timeout])
				continue;
			
			[aObj retain];
			[mAppls removeObjectAtIndex:i];
			[mApplsByName removeObjectForKey:[aObj name]];
			[aObj release];
			
			count++;
		}
	}
	
	return count;
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark accessor functions (misc)





- (int)count
{
	return [mAppls count];
}

- (int)sockCount
{
	return mSockCount;
}

- (NSArray *)getArray
{
	return mAppls;
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark data source functions (get)





- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [self count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	if (rowIndex >= [mAppls count])
		return nil;
	
	return [mAppls objectAtIndex:rowIndex];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark data source functions (set)





- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
}

- (BOOL)tableView:(NSTableView *)aTableView shouldEditTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	return FALSE;
}

- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	if (anObject == nil)
		return;
	
	if (rowIndex >= [mAppls count])
		return;
}

@end





///////////////////////////////////////////////////////////////////////////////

#pragma mark -





@implementation AM_ApplObject

- (id)initWithName:(NSString *)aStr origName:(NSString *)origName andCount:(uint32_t)anInt
{
	self = [super init];
	
	if (self) {
		mName = [aStr retain];
		mTimeout = 0;
		mCount = anInt;
		mIcon = [[GlowWorm getApplicationIcon:origName] retain];
		[mIcon setSize:NSSizeFromString(@"{30,30}")];
	}
	
	return self;
}

- (void)dealloc
{
	[mName release];
	[mIcon release];
	
	[super dealloc];
}

- (NSString *)name
{
	return mName;
}

- (uint32_t)count
{
	return mCount;
}

- (uint32_t)timeout
{
	return mTimeout;
}

- (NSImage *)icon
{
	return mIcon;
}

- (void)incrementCount
{
	@synchronized (self) {
		mCount++;
		mTimeout = 0;
	}
}

- (void)decrementCount
{
	@synchronized (self) {
		mCount--;
		
		if (mCount == 0) {
			struct timeval tv;
			gettimeofday(&tv, NULL);
			mTimeout = tv.tv_sec + 5;
		}
	}
}

@end





///////////////////////////////////////////////////////////////////////////////

#pragma mark -





@implementation AM_ApplCell

- (id)init
{
	self = [super init];
	
	if (self) {
		mNameCell = [[NSTextFieldCell alloc] init];
		[mNameCell setFont:[NSFont fontWithName:@"Lucida Grande" size:10.0]];
		[mNameCell setWraps:FALSE];
		
		mStatCell = [[NSTextFieldCell alloc] init];
		[mStatCell setFont:[NSFont fontWithName:@"Lucida Grande" size:9.0]];
		[mNameCell setWraps:FALSE];
		
		mIconCell = [[NSImageCell alloc] init];
		[mIconCell setImageAlignment:NSImageAlignLeft];
		[mIconCell setImageFrameStyle:NSImageFrameNone];
		[mIconCell setImageScaling:NSScaleProportionally];
		
		NSImage * icon = [[GlowWorm getGenericApplicationIcon] retain];
		[icon setSize:NSSizeFromString(@"{30,30}")];
		[mIconCell setImage:icon];
	}
	
	return self;
}

- (id)copyWithZone:(NSZone *)zone
{
	AM_ApplCell * aCell = [super copyWithZone:zone];
	
	aCell->mNameCell = [mNameCell copyWithZone:zone];
	aCell->mStatCell = [mStatCell copyWithZone:zone];
	aCell->mIconCell = [mIconCell copyWithZone:zone];
	
	return aCell;
}

- (void)dealloc
{
	[mNameCell release];
	[mStatCell release];
	[mIconCell release];
	
	[super dealloc];
}

- (void)setObjectValue:(id <NSCopying>)object
{
	AM_ApplObject * aObj = (AM_ApplObject *)object;
	
	[mNameCell setStringValue:[aObj name]];
	[mStatCell setStringValue:[NSString stringWithFormat:@"%d %@", [aObj count], ([aObj count] == 1 ? @"connection" : @"connections")]];
	[mIconCell setImage:[aObj icon]];
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	//
	// application icon
	//
	cellFrame.origin.x += 4;
	[mIconCell drawInteriorWithFrame:cellFrame inView:controlView];
	
	//
	// application name
	//
	cellFrame.origin.x += 36;
	cellFrame.origin.y += 5;
	[mNameCell drawInteriorWithFrame:cellFrame inView:controlView];
	
	//
	// statistics value
	//
	cellFrame.origin.y += 15;
	[mStatCell drawInteriorWithFrame:cellFrame inView:controlView];
}

@end
